package com.tuya.signature;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import static java.time.ZoneOffset.UTC;

/**
 * OpenAPI Signature Demo
 */
public class OpenAPISignatureDemo {
    public static void main(String[] args) {

        LocalDateTime currentDateTime = LocalDateTime.now(UTC);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss'Z'");
        String formattedDateTime = currentDateTime.format(formatter);

        String httpMethod = "GET";

        // tuya access key secret
        String accessKey = "xMr9wgwXQLhv5AUa65o03mcD";
        String secretKey = "dZQyYWPshkcHWQceilEA1CodQoKvrI1eCO7H9wAWxMW8P0G5";

        // host信息，格式： https://domain:port
        String host = "https://si-d6e8d812acb848958054.tuyacloud.com:8686";

        // action信息，即接口名字
        String action = "SetDeviceDesiredProperty";

        String signatureOnce = UUID.randomUUID().toString().replace("-", "");


        //方法一：已经有url，使用url进行签名，注意url里面不能有Signature参数
        String url = host+"/?Action="+action+"&ProductKey=dwwAsxAkOwT7xhIY&AccessKeyId=" + accessKey + "&SignatureMethod=HMAC-SHA1&Timestamp=" + formattedDateTime + "&SignatureNonce="+ signatureOnce;
        try {
            if(url.contains("Signature=")){
                System.out.println("url里面不能有Signature参数");
                return;
            }
            Map<String, String> queryMap = SignatureUtils.splitQueryString(url);
            String signature = SignatureUtils.generate(httpMethod, queryMap, secretKey);
            System.out.println("[方法一：开始生成]======根据url生成签名，并且生成curl命令");
            System.out.println("curl " + "'" + url + "&Signature=" + signature + "'");

        } catch (Exception e) {
            System.out.println("生成签名失败"+e.getMessage());
            e.printStackTrace();
        }

        //方法二：已经有参数Map，使用参数进行签名
        Map<String, String> map = new HashMap<String, String>();
        // 公共参数
        map.put("AccessKeyId", accessKey);
        map.put("SignatureMethod", "HMAC-SHA1");
        map.put("Timestamp", formattedDateTime);
        map.put("SignatureNonce", signatureOnce);
        // 接口请求参数
        map.put("Action", action);
        map.put("Versions", "{}");
        //map.put("IotInstanceId", "iot-06z00j616bccotp");
        map.put("Items", "{\"rain\":13}");
        map.put("ProductKey", "k1ymv1DuLfE");
        map.put("DeviceName", "a53b1fa6490e");
        try {
            String signature = SignatureUtils.generate(httpMethod, map, secretKey);
            System.out.println("[方法二:开始生成]======根据参数生成签名，并且生成curl命令");
            System.out.println("curl " + "'" + host+"/?"+buildQueryParam(map) + "&Signature=" + signature + "'");

        } catch (Exception e) {
            System.out.println("生成签名失败"+e.getMessage());
            e.printStackTrace();
        }

    }

    // 把map生成query param，添加转义处理
    private static String buildQueryParam(Map<String, String> queryMap) {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : queryMap.entrySet()) {
            try {
                // 对键和值分别进行URL编码转义
                String encodedKey = URLEncoder.encode(entry.getKey());
                String encodedValue = URLEncoder.encode(entry.getValue());
                sb.append(encodedKey).append("=").append(encodedValue).append("&");
            } catch (Exception e) {
                System.out.println("URL编码转义失败: " + e.getMessage());
                e.printStackTrace();
            }
        }
        //删除最后一个&
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }

}